/*****************************************************************************\
* (c) Copyright 2018-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the Apache License          *
* version 2 (Apache-2.0), copied verbatim in the file "LICENSE".              *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#pragma once

#include "UTDefinitions.cuh"
#include "VeloConsolidated.cuh"
#include "UTMagnetToolDefinitions.h"
#include "CompassUTDefinitions.cuh"
#include "AlgorithmTypes.cuh"

namespace ut_search_windows {
  struct Parameters {
    HOST_INPUT(host_number_of_events_t, unsigned) host_number_of_events;
    HOST_INPUT(host_number_of_reconstructed_velo_tracks_t, unsigned) host_number_of_reconstructed_velo_tracks;
    DEVICE_INPUT(dev_number_of_events_t, unsigned) dev_number_of_events;
    DEVICE_INPUT(dev_ut_hits_t, char) dev_ut_hits;
    DEVICE_INPUT(dev_ut_hit_offsets_t, unsigned) dev_ut_hit_offsets;
    DEVICE_INPUT(dev_velo_tracks_view_t, Allen::Views::Velo::Consolidated::Tracks) dev_velo_tracks_view;
    DEVICE_INPUT(dev_velo_states_view_t, Allen::Views::Physics::KalmanStates) dev_velo_states_view;
    DEVICE_INPUT(dev_ut_number_of_selected_velo_tracks_t, unsigned) dev_ut_number_of_selected_velo_tracks;
    DEVICE_INPUT(dev_ut_selected_velo_tracks_t, unsigned) dev_ut_selected_velo_tracks;
    MASK_INPUT(dev_event_list_t) dev_event_list;
    DEVICE_OUTPUT(dev_ut_windows_layers_t, short) dev_ut_windows_layers;
    PROPERTY(min_momentum_t, "min_momentum", "min momentum cut [MeV/c]", float) min_momentum;
    PROPERTY(min_pt_t, "min_pt", "min pT cut [MeV/c]", float) min_pt;
    PROPERTY(y_tol_t, "y_tol", "y tol [mm]", float) y_tol;
    PROPERTY(y_tol_slope_t, "y_tol_slope", "y tol slope [mm]", float) y_tol_slope;
    PROPERTY(block_dim_y_t, "block_dim_y_t", "block dimension Y", unsigned) block_dim_y;
  };

  __global__ void ut_search_windows(
    Parameters,
    UTMagnetTool* dev_ut_magnet_tool,
    const UT::Constants::PerLayerInfo* dev_mean_layer_info,
    const unsigned* dev_unique_x_sector_layer_offsets,
    const float* dev_unique_sector_xs);

  struct ut_search_windows_t : public DeviceAlgorithm, Parameters {
    void set_arguments_size(ArgumentReferences<Parameters> arguments, const RuntimeOptions&, const Constants&) const;

    void operator()(
      const ArgumentReferences<Parameters>& arguments,
      const RuntimeOptions&,
      const Constants& constants,
      const Allen::Context& context) const;

  private:
    Property<min_momentum_t> m_mom {this, 1500.f * Gaudi::Units::MeV};
    Property<min_pt_t> m_pt {this, 300.f * Gaudi::Units::MeV};
    Property<y_tol_t> m_ytol {this, 0.5f * Gaudi::Units::mm};
    Property<y_tol_slope_t> m_yslope {this, 0.08f};
    Property<block_dim_y_t> m_block_dim_y {this, 128};
  };
} // namespace ut_search_windows
