/*****************************************************************************\
* (c) Copyright 2018-2020 CERN for the benefit of the LHCb Collaboration      *
*                                                                             *
* This software is distributed under the terms of the Apache License          *
* version 2 (Apache-2.0), copied verbatim in the file "LICENSE".              *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
/** @file MCEvent.h
 *
 * @brief a reader of MC events
 *
 * @author Rainer Schwemmer
 * @author Daniel Campora
 * @author Manuel Schiller
 * @date 2018-02-18
 *
 * 2018-07 Dorothea vom Bruch: updated to run over different track types,
 * take input from Renato Quagliani's TrackerDumper
 */

#pragma once

#include <cassert>
#include <cstdint>
#include <string>
#include <vector>
#include <algorithm>
#include "MCParticle.h"
#include "Common.h"
#include "Logger.h"
#include "LHCbID.cuh"
#include "MCVertex.h"

struct MCEvent {

  MCVertices m_mcvs;
  MCParticles m_mcps;
  uint32_t size;

  // Constructor
  MCEvent() {}
  MCEvent(
    std::vector<char> const& _particles,
    std::vector<char> const& _vertices,
    const bool checkFile = true,
    const uint32_t bankVersion = 1);

  // Checks if a LHCb ID is in a particular subdetector
  bool is_subdetector_impl(const LHCbID (&array)[42], const LHCbID& id) const;

  // Subdetector-specialized check
  template<typename T>
  bool is_subdetector(const LHCbID& id) const;

  // Checks an MCP does not contain invalid values
  void check_mcp(const MCParticle& mcp);

private:
  void load_particles(std::vector<char> const& particles, const uint32_t bankVersion = 1);

  void load_vertices(std::vector<char> const& vertices, const uint32_t bankVersion = 1);
};

using MCEvents = std::vector<MCEvent>;
