/*****************************************************************************\
* (c) Copyright 2021 CERN for the benefit of the LHCb Collaboration           *
*                                                                             *
* This software is distributed under the terms of the Apache License          *
* version 2 (Apache-2.0), copied verbatim in the file "COPYING".              *
*                                                                             *
* In applying this licence, CERN does not waive the privileges and immunities *
* granted to it by virtue of its status as an Intergovernmental Organization  *
* or submit itself to any jurisdiction.                                       *
\*****************************************************************************/
#pragma once

#include "Logger.h"

#ifndef ALLEN_STANDALONE
#include <Gaudi/BaseSink.h>
#endif

#include <deque>

#ifdef ALLEN_STANDALONE
struct MonitoringPrinter {
  MonitoringPrinter(unsigned = 0, bool = true) {}
  void process(bool = false) {}
};
#else
struct MonitoringPrinter : public Gaudi::Monitoring::BaseSink {
  using Entity = Gaudi::Monitoring::Hub::Entity;

  MonitoringPrinter(std::string name, ISvcLocator* svcloc, unsigned int printPeriod = 10, bool do_print = true) :
    BaseSink(name, svcloc), m_printPeriod(printPeriod), m_print(do_print)
  {}

  void flush(bool) override
  {
    applyToAllEntities([](auto& entity) {
      nlohmann::json j = entity;
      info_cout << entity.component << ":" << entity.name;
      if (j.count("nEntries")) {
        info_cout << "\tEntries: " << j["nEntries"];
      }
      info_cout << std::endl;
    });
  }

  void process(bool forcePrint = false)
  {
    if (m_print && (++m_delayCount >= m_printPeriod || forcePrint)) {
      m_delayCount = 0;
      flush(false);
    }
  }

private:
  unsigned int m_printPeriod;
  unsigned int m_delayCount {0};
  bool m_print;
};
#endif
